from .rest import RestClient


class Clients(object):
    """Auth0 applications endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/clients'.format(self.protocol, self.domain)
        if id is not None:
            return '{}/{}'.format(url, id)
        return url

    def all(self, fields=None, include_fields=True, page=None, per_page=None, extra_params=None):
        """Retrieves a list of all the applications.

        Important: The client_secret and encryption_key attributes can only be
        retrieved with the read:client_keys scope.

        Args:
           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

           extra_params (dictionary, optional): The extra parameters to add to
             the request. The fields, include_fields, page and per_page values
             specified as parameters take precedence over the ones defined here.

        See: https://auth0.com/docs/api/management/v2#!/Clients/get_clients
        """
        params = extra_params or {}
        params['fields'] = fields and ','.join(fields) or None
        params['include_fields'] = str(include_fields).lower()
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(), params=params)

    def create(self, body):
        """Create a new application.

        Args:
           body (dict): Attributes for the new application.

        See: https://auth0.com/docs/api/v2#!/Clients/post_clients
        """

        return self.client.post(self._url(), data=body)

    def get(self, id, fields=None, include_fields=True):
        """Retrieves an application by its id.

        Important: The client_secret, encryption_key and signing_keys
        attributes can only be retrieved with the read:client_keys scope.

        Args:
           id (str): Id of the application to get.

           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

        See: https://auth0.com/docs/api/management/v2#!/Clients/get_clients_by_id
        """

        params = {'fields': fields and ','.join(fields) or None,
                  'include_fields': str(include_fields).lower()}

        return self.client.get(self._url(id), params=params)

    def delete(self, id):
        """Deletes an application and all its related assets.

        Args:
           id (str): Id of application to delete.

        See: https://auth0.com/docs/api/management/v2#!/Clients/delete_clients_by_id
        """

        return self.client.delete(self._url(id))

    def update(self, id, body):
        """Modifies an application.

        Important: The client_secret, encryption_key and signing_keys
        attributes can only be updated with the update:client_keys scope.

        Args:
           id (str): Client ID of the application.

           body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Clients/patch_clients_by_id
        """

        return self.client.patch(self._url(id), data=body)

    def rotate_secret(self, id):
        """Rotate a client secret. The generated secret is NOT base64 encoded.

        Args:
           id (str): Client ID of the application.

        See: https://auth0.com/docs/api/management/v2#!/Clients/post_rotate_secret
        """

        data = {'id': id}

        url = self._url('%s/rotate-secret' % id)
        return self.client.post(url, data=data)



import unittest

import mock

from ...management.log_streams import LogStreams


class TestLogStreams(unittest.TestCase):

    def test_init_with_optionals(self):
        t = LogStreams(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.log_streams.RestClient')
    def test_list(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain='domain', token='jwttoken')

        c.list()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/log-streams', args[0])
        
    @mock.patch('auth0.v3.management.log_streams.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain='domain', token='jwttoken')
        c.get('an-id')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/log-streams/an-id', args[0])

    @mock.patch('auth0.v3.management.log_streams.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain='domain', token='jwttoken')
        # Sample data belongs to an `http` stream
        log_stream_data = {
            "name": "string",
            "type": "http",
            "sink": {
                "httpEndpoint": "string",
                "httpContentType": "string",
                "httpContentFormat": "JSONLINES|JSONARRAY",
                "httpAuthorization": "string"
            }
        }
        c.create(log_stream_data)

        args, kwargs = mock_instance.post.call_args

        self.assertEqual('https://domain/api/v2/log-streams', args[0])
        self.assertEqual(kwargs['data'], log_stream_data)

    @mock.patch('auth0.v3.management.log_streams.RestClient')
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain='domain', token='jwttoken')
        c.delete('an-id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/log-streams/an-id'
        )

    @mock.patch('auth0.v3.management.log_streams.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value
        log_stream_update = {
            "name": "string"
        }

        c = LogStreams(domain='domain', token='jwttoken')
        c.update('an-id', log_stream_update)

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual('https://domain/api/v2/log-streams/an-id', args[0])
        self.assertEqual(kwargs['data'], log_stream_update)



import unittest
import mock
from ...authentication.passwordless import Passwordless


class TestPasswordless(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_email(self, mock_post):

        p = Passwordless('my.domain.com')

        p.email(client_id='cid',
                email='a@b.com',
                send='snd')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/passwordless/start')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'email': 'a@b.com',
            'send': 'snd',
            'connection': 'email',
        })

    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_email_with_auth_params(self, mock_post):

        p = Passwordless('my.domain.com')

        p.email(client_id='cid',
                email='a@b.com',
                send='snd',
                auth_params={'a': 'b'})

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/passwordless/start')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'email': 'a@b.com',
            'send': 'snd',
            'authParams': {'a': 'b'},
            'connection': 'email',
        })

    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_email_with_client_secret(self, mock_post):

        p = Passwordless('my.domain.com')

        p.email(client_id='cid',
                client_secret='csecret',
                email='a@b.com',
                send='snd')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/passwordless/start')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'client_secret': 'csecret',
            'email': 'a@b.com',
            'send': 'snd',
            'connection': 'email',
        })

    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_sms(self, mock_post):
        p = Passwordless('my.domain.com')

        p.sms(client_id='cid', phone_number='123456')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/passwordless/start')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'phone_number': '123456',
            'connection': 'sms',
        })

    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_sms_with_client_secret(self, mock_post):
        p = Passwordless('my.domain.com')

        p.sms(client_id='cid', client_secret='csecret', phone_number='123456')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/passwordless/start')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'client_secret': 'csecret',
            'phone_number': '123456',
            'connection': 'sms',
        })
        
    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_sms_login(self, mock_post):

        p = Passwordless('my.domain.com')

        p.sms_login(client_id='cid', phone_number='123456', code='abcd')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/ro')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'connection': 'sms',
            'grant_type': 'password',
            'username': '123456',
            'password': 'abcd',
            'scope': 'openid',
        })

    @mock.patch('auth0.v3.authentication.passwordless.Passwordless.post')
    def test_sms_login_with_scope(self, mock_post):

        p = Passwordless('my.domain.com')

        p.sms_login(client_id='cid', phone_number='123456',
                    code='abcd', scope='openid profile')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/ro')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'connection': 'sms',
            'grant_type': 'password',
            'username': '123456',
            'password': 'abcd',
            'scope': 'openid profile',
        })


from .base import AuthenticationBase
try:
    from urllib.parse import quote_plus
except ImportError:
    from urllib import quote_plus


class Logout(AuthenticationBase):

    """Logout Endpoint

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def logout(self, client_id, return_to, federated=False):
        """Logout

        Use this endpoint to logout a user. If you want to navigate the user to a
        specific URL after the logout, set that URL at the returnTo parameter.
        The URL should be included in any the appropriate Allowed Logout URLs list:

        Args:
            client_id (str): The client_id of your application.

            returnTo (str): URL to redirect the user after the logout.

            federated (bool): Querystring parameter to log the user out of the IdP
        """
        return_to = quote_plus(return_to)

        if federated is True:
            return self.get(
                '{}://{}/v2/logout?federated&client_id={}&returnTo={}'.format(
                    self.protocol, self.domain, client_id, return_to)
            )
        return self.get(
            '{}://{}/v2/logout?client_id={}&returnTo={}'.format(self.protocol,
                                                                   self.domain,
                                                                   client_id,
                                                                   return_to)
        )



import unittest
import mock
from ...management.blacklists import Blacklists


class TestBlacklists(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Blacklists(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.blacklists.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        t = Blacklists(domain='domain', token='jwttoken')
        t.get(aud='an-id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/blacklists/tokens',
            params={'aud': 'an-id'}
        )

    @mock.patch('auth0.v3.management.blacklists.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        t = Blacklists(domain='domain', token='jwttoken')

        # create without audience
        t.create(jti='the-jti')

        args, kwargs = mock_instance.post.call_args

        self.assertEqual('https://domain/api/v2/blacklists/tokens', args[0])
        self.assertEqual(kwargs['data'], {'jti': 'the-jti'})

        # create with audience
        t.create(jti='the-jti', aud='the-aud')

        args, kwargs = mock_instance.post.call_args

        self.assertEqual('https://domain/api/v2/blacklists/tokens', args[0])
        self.assertEqual(kwargs['data'], {'jti': 'the-jti', 'aud': 'the-aud'})



from .rest import RestClient


class Connections(object):
    """Auth0 connection endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/connections'.format(self.protocol, self.domain)
        if id is not None:
            return '{}/{}'.format(url, id)
        return url

    def all(self, strategy=None, fields=None, include_fields=True, page=None, per_page=None, extra_params=None):
        """Retrieves all connections.

        Args:
           strategy (str, optional): Only retrieve connections of
              this strategy type. (e.g: strategy='amazon')

           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields. By default, all the fields will be retrieved.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

           extra_params (dictionary, optional): The extra parameters to add to
             the request. The fields, include_fields, page and per_page values
             specified as parameters take precedence over the ones defined here.

        See: https://auth0.com/docs/api/management/v2#!/Connections/get_connections

        Returns:
           A list of connection objects.
        """

        params = extra_params or {}
        params['strategy'] = strategy or None
        params['fields'] = fields and ','.join(fields) or None
        params['include_fields'] = str(include_fields).lower()
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(), params=params)

    def get(self, id, fields=None, include_fields=True):
        """Retrieve connection by id.

        Args:
           id (str): Id of the connection to get.

           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields. By default, all the fields will be retrieved.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

        See: https://auth0.com/docs/api/management/v2#!/Connections/get_connections_by_id

        Returns:
            A connection object.
        """

        params = {'fields': fields and ','.join(fields) or None,
                  'include_fields': str(include_fields).lower()}

        return self.client.get(self._url(id), params=params)

    def delete(self, id):
        """Deletes a connection and all its users.

        Args:
           id: Id of the connection to delete.

        See: https://auth0.com/docs/api/management/v2#!/Connections/delete_connections_by_id

        Returns:
           An empty dict.
        """

        return self.client.delete(self._url(id))

    def update(self, id, body):
        """Modifies a connection.

        Args:
           id: Id of the connection.

           body (dict): Specifies which fields are to be modified, and to what values.

        See: https://auth0.com/docs/api/management/v2#!/Connections/patch_connections_by_id

        Returns:
           The modified connection object.
        """

        return self.client.patch(self._url(id), data=body)

    def create(self, body):
        """Creates a new connection.

        Args:
            body (dict): Attributes used to create the connection. Mandatory
                attributes are: 'name' and 'strategy'.

        See: https://auth0.com/docs/api/management/v2#!/Connections/post_connections
        """

        return self.client.post(self._url(), data=body)

    def delete_user_by_email(self, id, email):
        """Deletes a specified connection user by its email.

        Args:
           id (str): The id of the connection (must be a database connection).

           email (str): The email of the user to delete.

        See: https://auth0.com/docs/api/management/v2#!/Connections/delete_users_by_email

        Returns:
            An empty dict.
        """
        return self.client.delete(self._url(id) + '/users', params={'email': email})



from .rest import RestClient


class Guardian(object):
    """Auth0 guardian endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/guardian'.format(self.protocol, self.domain)
        if id is not None:
            return '{}/{}'.format(url, id)
        return url

    def all_factors(self):
        """Retrieves all factors. Useful to check factor enablement and
             trial status.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/get_factors                 
        """

        return self.client.get(self._url('factors'))

    def update_factor(self, name, body):
        """Update Guardian factor.
        Useful to enable / disable factor.

        Args:
            name (str): Either push-notification or sms.

            body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/put_factors_by_name
        """
        url = self._url('factors/{}'.format(name))
        return self.client.put(url, data=body)

    def update_templates(self, body):
        """Update enrollment and verification SMS templates.

        Useful to send custom messages on sms enrollment and verification.

        Args:
            body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/put_templates
        """

        return self.client.put(self._url('factors/sms/templates'), data=body)

    def get_templates(self):
        """Get enrollment and verification templates.

        Retrieve both templates. Useful to check if a different template than
            default was set.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/get_templates
        """

        return self.client.get(self._url('factors/sms/templates'))

    def get_enrollment(self, id):
        """Retrieves an enrollment.
        Useful to check its type and related metadata.

        Args:
           id (str): The id of the device account to update.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/get_enrollments_by_id
        """
        url = self._url('enrollments/{}'.format(id))
        return self.client.get(url)

    def delete_enrollment(self, id):
        """Deletes an enrollment.

        Useful when you want to force re-enroll.

        Args:
           id (str): The id of the device account to update.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/delete_enrollments_by_id
        """
        url = self._url('enrollments/{}'.format(id))
        return self.client.delete(url)

    def create_enrollment_ticket(self, body):
        """Creates an enrollment ticket for user_id

        A useful way to send an email to a user, with a link that lead to
            start the enrollment process.

        Args:
            body (dict): Details of the user to send the ticket to.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/post_ticket
        """
        return self.client.post(self._url('enrollments/ticket'), data=body)

    def get_factor_providers(self, factor_name, name):
        """Get Guardian SNS or SMS factor providers.

        Returns provider configuration.

        Args:
           factor_name (str): Either push-notification or sms.

           name (str): Name of the provider.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/get_sns
             https://auth0.com/docs/api/management/v2#!/Guardian/get_twilio
        """
        url = self._url('factors/{}/providers/{}'.format(factor_name, name))
        return self.client.get(url)

    def update_factor_providers(self, factor_name, name, body):
        """Get Guardian factor providers.

        Returns provider configuration.

        Args:
           factor_name (str): Either push-notification or sms.

           name (str): Name of the provider.

           body (dict): Details of the factor provider.

        See: https://auth0.com/docs/api/management/v2#!/Guardian/put_twilio
        """
        url = self._url('factors/{}/providers/{}'.format(factor_name, name))
        return self.client.put(url, data=body)



import unittest
import mock
from ...management.connections import Connections


class TestConnection(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Connections(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.connections.RestClient')
    def test_all(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        # Default parameters are requested
        c = Connections(domain='domain', token='jwttoken')
        c.all()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/connections', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'strategy': None,
                                            'page': None,
                                            'per_page': None,
                                            'include_fields': 'true'})

        # Fields filter
        c.all(fields=['a', 'b'], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/connections', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'strategy': None,
                                            'page': None,
                                            'per_page': None,
                                            'include_fields': 'false'})

        # Fields + strategy filter
        c.all(fields=['a', 'b'], strategy='auth0', include_fields=True)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/connections', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'strategy': 'auth0',
                                            'page': None,
                                            'per_page': None,
                                            'include_fields': 'true'})

        # Specific pagination
        c.all(page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/connections', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'strategy': None,
                                            'page': 7,
                                            'per_page': 25,
                                            'include_fields': 'true'})

        # Extra parameters
        c.all(extra_params={'some_key': 'some_value'})

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/connections', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'strategy': None,
                                            'page': None,
                                            'per_page': None,
                                            'include_fields': 'true',
                                            'some_key': 'some_value'})

    @mock.patch('auth0.v3.management.connections.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        c = Connections(domain='domain', token='jwttoken')
        c.get('an-id')

        args, kwargs = mock_instance.get.call_args
        self.assertEqual('https://domain/api/v2/connections/an-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true'})

        c.get('an-id', fields=['a', 'b'])

        args, kwargs = mock_instance.get.call_args
        self.assertEqual('https://domain/api/v2/connections/an-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'true'})

        c.get('an-id', fields=['a', 'b'], include_fields=False)

        args, kwargs = mock_instance.get.call_args
        self.assertEqual('https://domain/api/v2/connections/an-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'false'})

    @mock.patch('auth0.v3.management.connections.RestClient')
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.delete.return_value = {}

        c = Connections(domain='domain', token='jwttoken')
        c.delete('this-id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/connections/this-id'
        )

    @mock.patch('auth0.v3.management.connections.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.patch.return_value = {}

        c = Connections(domain='domain', token='jwttoken')
        c.update('that-id', {'a': 'b', 'c': 'd'})

        mock_instance.patch.assert_called_with(
            'https://domain/api/v2/connections/that-id',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.connections.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.post.return_value = {}

        c = Connections(domain='domain', token='jwttoken')
        c.create({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/connections',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.connections.RestClient')
    def test_delete_user_by_email(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.delete_user_by_email.return_value = {}

        c = Connections(domain='domain', token='jwttoken')
        c.delete_user_by_email('123', 'test@example.com')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/connections/123/users',
            params={'email': 'test@example.com'})



import unittest
import mock
from ...management.guardian import Guardian


class TestGuardian(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Guardian(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_all_factors(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.all_factors()

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/guardian/factors'
        )

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_update_factor(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.update_factor('push-notification', {'enabled': True})

        args, kwargs = mock_instance.put.call_args
        self.assertEqual('https://domain/api/v2/guardian/factors/push-notification', args[0])
        self.assertEqual(kwargs['data'], {'enabled': True})

        g.update_factor('sms', {'enabled': False})

        args, kwargs = mock_instance.put.call_args
        self.assertEqual('https://domain/api/v2/guardian/factors/sms', args[0])
        self.assertEqual(kwargs['data'], {'enabled': False})

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_update_templates(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.update_templates({'enrollment_message': 'hello',
                            'verification_message': 'verified'})

        args, kwargs = mock_instance.put.call_args
        self.assertEqual('https://domain/api/v2/guardian/factors/sms/templates', args[0])
        self.assertEqual(kwargs['data'], {'enrollment_message': 'hello',
                                          'verification_message': 'verified'})

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_get_templates(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.get_templates()

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/guardian/factors/sms/templates'
        )

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_get_enrollment(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.get_enrollment('some_id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/guardian/enrollments/some_id'
        )

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_delete_enrollment(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.delete_enrollment('some_id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/guardian/enrollments/some_id'
        )

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_create_enrollment_ticket(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.create_enrollment_ticket({'user_id': 'some_id',
                                    'email': 'test@test.com',
                                    'send_mail': 'false'})

        args, kwargs = mock_instance.post.call_args
        self.assertEqual('https://domain/api/v2/guardian/enrollments/ticket', args[0])
        self.assertEqual(kwargs['data'], {'user_id': 'some_id',
                                          'email': 'test@test.com',
                                          'send_mail': 'false'})

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_get_factor_providers(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.get_factor_providers('sms', 'twilio')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/guardian/factors/sms/providers/twilio'
        )

    @mock.patch('auth0.v3.management.guardian.RestClient')
    def test_update_factor_providers(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain='domain', token='jwttoken')
        g.update_factor_providers('sms',
                                  'twilio',
                                  {'from': 'test@test.com',
                                   'messaging_service_sid': 'qwerty',
                                   'auth_token': 'abc.xyz.123',
                                   'sid': 'abc.xyz'})

        args, kwargs = mock_instance.put.call_args
        self.assertEqual('https://domain/api/v2/guardian/factors/sms/providers/twilio', args[0])
        self.assertEqual(kwargs['data'], {'from': 'test@test.com',
                                          'messaging_service_sid': 'qwerty',
                                          'auth_token': 'abc.xyz.123',
                                          'sid': 'abc.xyz'})



import unittest
import mock
from ...management.clients import Clients


class TestClients(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Clients(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.clients.RestClient')
    def test_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Clients(domain='domain', token='jwttoken')

        # Default parameters are requested
        c.all()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/clients', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true',
                                            'page': None,
                                            'per_page': None})

        # Fields filter
        c.all(fields=['a', 'b'], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/clients', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'false',
                                            'page': None,
                                            'per_page': None})

        # Specific pagination
        c.all(page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/clients', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true',
                                            'page': 7,
                                            'per_page': 25})

        # Extra parameters
        c.all(extra_params={'some_key': 'some_value'})

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/clients', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true',
                                            'page': None,
                                            'per_page': None,
                                            'some_key': 'some_value'})

    @mock.patch('auth0.v3.management.clients.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Clients(domain='domain', token='jwttoken')
        c.create({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/clients',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.clients.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Clients(domain='domain', token='jwttoken')
        c.get('this-id')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/clients/this-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true'})

        c.get('this-id', fields=['a', 'b'], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/clients/this-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'false'})

    @mock.patch('auth0.v3.management.clients.RestClient')
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Clients(domain='domain', token='jwttoken')
        c.delete('this-id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/clients/this-id'
        )

    @mock.patch('auth0.v3.management.clients.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Clients(domain='domain', token='jwttoken')
        c.update('this-id', {'a': 'b', 'c': 'd'})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual('https://domain/api/v2/clients/this-id', args[0])
        self.assertEqual(kwargs['data'], {'a': 'b', 'c': 'd'})

    @mock.patch('auth0.v3.management.clients.RestClient')
    def test_rotate_secret(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Clients(domain='domain', token='jwttoken')
        c.rotate_secret('this-id')

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/clients/this-id/rotate-secret', data={'id': 'this-id'}
        )






from .rest import RestClient


class Roles(object):
    """Auth0 roles endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/roles'.format(self.protocol, self.domain)
        if id is not None:
            return '{}/{}'.format(url, id)
        return url

    def list(self, page=0, per_page=25, include_totals=True, name_filter=None):
        """List or search roles.

        Args:
            page (int, optional): The result's page number (zero based). By default,
               retrieves the first page of results.

            per_page (int, optional): The amount of entries per page. By default,
               retrieves 25 results per page.

            include_totals (bool, optional): True if the query summary is
                to be included in the result, False otherwise. Defaults to True.

            name_filter (str, optional): A case-insensitive filter to apply
                to search for roles by name.

        See: https://auth0.com/docs/api/management/v2#!/Roles/get_roles
        """
        params = {
            'per_page': per_page,
            'page': page,
            'include_totals': str(include_totals).lower(),
            'name_filter': name_filter
        }
        return self.client.get(self._url(), params=params)

    def create(self, body):
        """Creates a new role.

        Args:
            body (dict): the attributes for the role to create.

        See: https://auth0.com/docs/api/v2#!/Roles/post_roles
        """
        return self.client.post(self._url(), data=body)

    def get(self, id):
        """Get a role.

        Args:
            id (str): The id of the role to retrieve.

        See: https://auth0.com/docs/api/management/v2#!/Roles/get_roles_by_id
        """

        return self.client.get(self._url(id))

    def delete(self, id):
        """Delete a role.

        Args:
            id (str): The id of the role to delete.

        See: https://auth0.com/docs/api/management/v2#!/Roles/delete_roles_by_id
        """
        return self.client.delete(self._url(id))

    def update(self, id, body):
        """Update a role with the attributes passed in 'body'

        Args:
            id (str): The id of the role to update.

            body (dict): the attributes to update on the role.

        See: https://auth0.com/docs/api/management/v2#!/Roles/patch_roles_by_id
        """
        return self.client.patch(self._url(id), data=body)

    def list_users(self, id, page=0, per_page=25, include_totals=True):
        """List the users that have been associated with a given role.

        Args:
            id (str): The role's id.

            page (int, optional): The result's page number (zero based). By default,
               retrieves the first page of results.

            per_page (int, optional): The amount of entries per page. By default,
               retrieves 25 results per page.

            include_totals (bool, optional): True if the query summary is
                to be included in the result, False otherwise. Defaults to True.

        See https://auth0.com/docs/api/management/v2#!/Roles/get_role_user
        """

        params = {
            'per_page': per_page,
            'page': page,
            'include_totals': str(include_totals).lower()
        }
        url = self._url('{}/users'.format(id))
        return self.client.get(url, params=params)

    def add_users(self, id, users):
        """Assign users to a role.

        Args:
            id (str): The role's id.

            users (list of str): A list of users ids to add to this role.

        See https://auth0.com/docs/api/management/v2#!/Roles/post_role_users
        """
        url = self._url('{}/users'.format(id))
        body = {'users': users}
        return self.client.post(url, data=body)

    def list_permissions(self, id, page=0, per_page=25, include_totals=True):
        """List the permissions associated to a role.

        Args:
            id (str): The role's id.

            page (int, optional): The result's page number (zero based). By default,
               retrieves the first page of results.

            per_page (int, optional): The amount of entries per page. By default,
               retrieves 25 results per page.

            include_totals (bool, optional): True if the query summary is
                to be included in the result, False otherwise. Defaults to True.

        See https://auth0.com/docs/api/management/v2#!/Roles/get_role_permission
        """
        params = {
            'per_page': per_page,
            'page': page,
            'include_totals': str(include_totals).lower()
        }
        url = self._url('{}/permissions'.format(id))
        return self.client.get(url, params=params)

    def remove_permissions(self, id, permissions):
        """Unassociates permissions from a role.

        Args:
            id (str): The role's id.

            permissions (list of str): A list of permission ids to unassociate from the role.

        See https://auth0.com/docs/api/management/v2#!/Roles/delete_role_permission_assignment
        """
        url = self._url('{}/permissions'.format(id))
        body = {'permissions': permissions}
        return self.client.delete(url, data=body)

    def add_permissions(self, id, permissions):
        """Associates permissions with a role.

        Args:
            id (str): The role's id.

            permissions (list of str): A list of permission ids to associate to the role.

        See https://auth0.com/docs/api/management/v2#!/Roles/post_role_permission_assignment
        """
        url = self._url('{}/permissions'.format(id))
        body = {'permissions': permissions}
        return self.client.post(url, data=body)



import unittest
import mock
from ...management.organizations import Organizations


class TestOrganizations(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Organizations(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    # Organizations
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_all_organizations(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')

        # Default parameters are requested
        c.all_organizations()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations', args[0])
        self.assertEqual(kwargs['params'], {'page': None,
                                            'per_page': None})

        # Specific pagination
        c.all_organizations(page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations', args[0])
        self.assertEqual(kwargs['params'], {'page': 7,
                                            'per_page': 25})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_get_organization_by_name(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.get_organization_by_name('test-org')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/name/test-org', args[0])
        self.assertEqual(kwargs['params'], {})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_get_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.get_organization('org123')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/org123', args[0])
        self.assertEqual(kwargs['params'], {})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_create_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.create_organization({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/organizations',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_update_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.update_organization('this-id', {'a': 'b', 'c': 'd'})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual('https://domain/api/v2/organizations/this-id', args[0])
        self.assertEqual(kwargs['data'], {'a': 'b', 'c': 'd'})
   
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_delete_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.delete_organization('this-id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/organizations/this-id'
        )
    
    # Organization Connections
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_all_organization_connections(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')

        # Default parameters are requested
        c.all_organization_connections('test-org')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/enabled_connections', args[0])
        self.assertEqual(kwargs['params'], {'page': None,
                                            'per_page': None})

        # Specific pagination
        c.all_organization_connections('test-org', page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/enabled_connections', args[0])
        self.assertEqual(kwargs['params'], {'page': 7,
                                            'per_page': 25})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_get_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.get_organization_connection('test-org', 'test-con')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/enabled_connections/test-con', args[0])
        self.assertEqual(kwargs['params'], {})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_create_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.create_organization_connection('test-org', {'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/organizations/test-org/enabled_connections',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_update_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.update_organization_connection('test-org', 'test-con', {'a': 'b', 'c': 'd'})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/enabled_connections/test-con', args[0])
        self.assertEqual(kwargs['data'], {'a': 'b', 'c': 'd'})
   
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_delete_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.delete_organization_connection('test-org', 'test-con')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/organizations/test-org/enabled_connections/test-con'
        )

    # Organization Members
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_all_organization_members(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')

        # Default parameters are requested
        c.all_organization_members('test-org')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/members', args[0])
        self.assertEqual(kwargs['params'], {'page': None,
                                            'per_page': None})

        # Specific pagination
        c.all_organization_members('test-org', page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/members', args[0])
        self.assertEqual(kwargs['params'], {'page': 7,
                                            'per_page': 25})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_create_organization_members(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.create_organization_members('test-org', {'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/organizations/test-org/members',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_delete_organization_members(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.delete_organization_members('test-org', {'a': 'b', 'c': 'd'})

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/organizations/test-org/members',
            data={'a': 'b', 'c': 'd'}
        )

    # Organization Member Roles
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_all_organization_member_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')

        # Default parameters are requested
        c.all_organization_member_roles('test-org', 'test-user')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/members/test-user/roles', args[0])
        self.assertEqual(kwargs['params'], {'page': None,
                                            'per_page': None})

        # Specific pagination
        c.all_organization_member_roles('test-org', 'test-user', page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/members/test-user/roles', args[0])
        self.assertEqual(kwargs['params'], {'page': 7,
                                            'per_page': 25})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_create_organization_member_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.create_organization_member_roles('test-org', 'test-user', {'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/organizations/test-org/members/test-user/roles',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_delete_organization_member_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.delete_organization_member_roles('test-org', 'test-user', {'a': 'b', 'c': 'd'})

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/organizations/test-org/members/test-user/roles',
            data={'a': 'b', 'c': 'd'}
        )

    # Organization Invitations
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_all_organization_invitations(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')

        # Default parameters are requested
        c.all_organization_invitations('test-org')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/invitations', args[0])
        self.assertEqual(kwargs['params'], {'page': None,
                                            'per_page': None})

        # Specific pagination
        c.all_organization_invitations('test-org', page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/invitations', args[0])
        self.assertEqual(kwargs['params'], {'page': 7,
                                            'per_page': 25})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_get_organization_invitation(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.get_organization_invitation('test-org', 'test-inv')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/organizations/test-org/invitations/test-inv', args[0])
        self.assertEqual(kwargs['params'], {})

    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_create_organization_invitation(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.create_organization_invitation('test-org', {'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/organizations/test-org/invitations',
            data={'a': 'b', 'c': 'd'}
        )
   
    @mock.patch('auth0.v3.management.organizations.RestClient')
    def test_delete_organization_invitation(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain='domain', token='jwttoken')
        c.delete_organization_invitation('test-org', 'test-inv')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/organizations/test-org/invitations/test-inv'
        )



from .base import AuthenticationBase


class Enterprise(AuthenticationBase):

    """Enterprise endpoints.

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def saml_metadata(self, client_id):
        """Get SAML2.0 Metadata.

        Args:
            client_id (str): Client Id of the application to get the SAML metadata for.
        """

        return self.get(url='{}://{}/samlp/metadata/{}'.format(self.protocol, self.domain, client_id))

    def wsfed_metadata(self):
        """Returns the WS-Federation Metadata.
        """

        url = '{}://{}/wsfed/FederationMetadata' \
              '/2007-06/FederationMetadata.xml'

        return self.get(url=url.format(self.protocol, self.domain))



import unittest

import mock

from ...authentication.database import Database


class TestDatabase(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.database.Database.post')
    def test_login(self, mock_post):
        d = Database('my.domain.com')

        d.login(client_id='cid',
                username='usrnm',
                password='pswd',
                id_token='idt',
                connection='conn',
                device='dev',
                grant_type='gt',
                scope='openid profile')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/ro')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'username': 'usrnm',
            'password': 'pswd',
            'id_token': 'idt',
            'connection': 'conn',
            'device': 'dev',
            'grant_type': 'gt',
            'scope': 'openid profile',
        })

    @mock.patch('auth0.v3.authentication.database.Database.post')
    def test_signup(self, mock_post):
        d = Database('my.domain.com')

        # using only email and password
        d.signup(client_id='cid',
                 email='a@b.com',
                 password='pswd',
                 connection='conn')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/dbconnections/signup')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'email': 'a@b.com',
            'password': 'pswd',
            'connection': 'conn',
        })

        # Using also optional properties
        sample_meta = {
            'hobby': 'surfing',
            'preference': {
                'color': 'pink'
            }
        }
        d.signup(client_id='cid',
                 email='a@b.com',
                 password='pswd',
                 connection='conn',
                 username='usr',
                 user_metadata=sample_meta,
                 given_name='john',
                 family_name='doe',
                 name='john doe',
                 nickname='johnny',
                 picture='avatars.com/john-doe')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/dbconnections/signup')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'email': 'a@b.com',
            'password': 'pswd',
            'connection': 'conn',
            'username': 'usr',
            'user_metadata': sample_meta,
            'given_name': 'john',
            'family_name': 'doe',
            'name': 'john doe',
            'nickname': 'johnny',
            'picture': 'avatars.com/john-doe',
        })

    @mock.patch('auth0.v3.authentication.database.Database.post')
    def test_change_password(self, mock_post):
        d = Database('my.domain.com')

        # ignores the password argument
        d.change_password(client_id='cid',
                          email='a@b.com',
                          password='pswd',
                          connection='conn')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0],
                         'https://my.domain.com/dbconnections/change_password')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'email': 'a@b.com',
            'connection': 'conn',
        })



import unittest
import mock
from ...authentication.revoke_token import RevokeToken


class TestRevokeToken(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.revoke_token.RevokeToken.post')
    def test_revoke_refresh_token(self, mock_post):

        a = RevokeToken('my.domain.com')

        # regular apps
        a.revoke_refresh_token(client_id='cid', token='tkn')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/revoke')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'token': 'tkn'
        })

        # confidential apps
        a.revoke_refresh_token(client_id='cid',
                    token='tkn',
                    client_secret='sh!')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/revoke')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'token': 'tkn',
            'client_secret': 'sh!'
        })



import unittest
import mock
from ...authentication.authorize_client import AuthorizeClient


class TestAuthorizeClient(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.authorize_client.AuthorizeClient.get')
    def test_login(self, mock_get):

        a = AuthorizeClient('my.domain.com')

        a.authorize(client_id='cid',
                    audience='https://test.com/api',
                    state='st',
                    redirect_uri='http://localhost',
                    response_type='token',
                    scope='openid profile',
                    organization='org_123',
                    invitation='invitation_abc')

        args, kwargs = mock_get.call_args

        self.assertEqual(args[0], 'https://my.domain.com/authorize')
        self.assertEqual(kwargs['params'], {
            'client_id': 'cid',
            'audience': 'https://test.com/api',
            'state': 'st',
            'redirect_uri': 'http://localhost',
            'response_type': 'token',
            'scope': 'openid profile',
            'organization': 'org_123',
            'invitation': 'invitation_abc'
        })

    @mock.patch('auth0.v3.authentication.authorize_client.AuthorizeClient.get')
    def test_login_default_param_values(self, mock_get):

        a = AuthorizeClient('my.domain.com')

        a.authorize(client_id='cid')

        args, kwargs = mock_get.call_args

        self.assertEqual(args[0], 'https://my.domain.com/authorize')
        self.assertEqual(kwargs['params'], {
            'audience': None,
            'invitation': None,
            'organization':  None,
            'redirect_uri': None,
            'state': None,
            'client_id': 'cid',
            'response_type':  'code',
            'scope': 'openid'
        })



import unittest
import mock
from ...management.tickets import Tickets


class TestTickets(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Tickets(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.tickets.RestClient')
    def test_email(self, mock_rc):
        mock_instance = mock_rc.return_value

        t = Tickets(domain='domain', token='jwttoken')
        t.create_email_verification({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/tickets/email-verification',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.tickets.RestClient')
    def test_pswd(self, mock_rc):
        mock_instance = mock_rc.return_value

        t = Tickets(domain='domain', token='jwttoken')
        t.create_pswd_change({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/tickets/password-change',
            data={'a': 'b', 'c': 'd'}
        )



import unittest
import mock
from ...authentication.social import Social


class TestSocial(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.social.Social.post')
    def test_login(self, mock_post):
        s = Social('a.b.c')
        s.login(client_id='cid', access_token='atk', connection='conn')

        args, kwargs = mock_post.call_args

        self.assertEqual('https://a.b.c/oauth/access_token', args[0])
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'access_token': 'atk',
            'connection': 'conn',
            'scope': 'openid',
        })

    @mock.patch('auth0.v3.authentication.social.Social.post')
    def test_login_with_scope(self, mock_post):
        s = Social('a.b.c')
        s.login(client_id='cid', access_token='atk',
                connection='conn', scope='openid profile')

        args, kwargs = mock_post.call_args

        self.assertEqual('https://a.b.c/oauth/access_token', args[0])
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'access_token': 'atk',
            'connection': 'conn',
            'scope': 'openid profile',
        })



import unittest
import mock
from ...management.client_grants import ClientGrants


class TestClientGrants(unittest.TestCase):

    def test_init_with_optionals(self):
        t = ClientGrants(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.client_grants.RestClient')
    def test_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain='domain', token='jwttoken')

        # With default params
        c.all()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/client-grants', args[0])
        self.assertEqual(kwargs['params'], {
            'audience': None,
            'page': None,
            'per_page': None,
            'include_totals': 'false',
            'client_id': None,
        })

        # With audience
        c.all(audience='http://domain.auth0.com/api/v2/')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/client-grants', args[0])
        self.assertEqual(kwargs['params'], {
            'audience': 'http://domain.auth0.com/api/v2/',
            'page': None,
            'per_page': None,
            'include_totals': 'false',
            'client_id': None,
        })

        # With pagination params
        c.all(per_page=23, page=7, include_totals=True)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/client-grants', args[0])
        self.assertEqual(kwargs['params'], {
            'audience': None,
            'page': 7,
            'per_page': 23,
            'include_totals': 'true',
            'client_id': None,
        })

        # With client_id param
        c.all(client_id='exampleid')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/client-grants', args[0])
        self.assertEqual(kwargs['params'], {
            'audience': None,
            'page': None,
            'per_page': None,
            'include_totals': 'false',
            'client_id': 'exampleid',
        })

    @mock.patch('auth0.v3.management.client_grants.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain='domain', token='jwttoken')
        c.create({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/client-grants',
            data={'a': 'b', 'c': 'd'}
        )

    @mock.patch('auth0.v3.management.client_grants.RestClient')
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain='domain', token='jwttoken')
        c.delete('this-id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/client-grants/this-id'
        )

    @mock.patch('auth0.v3.management.client_grants.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain='domain', token='jwttoken')
        c.update('this-id', {'a': 'b', 'c': 'd'})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual('https://domain/api/v2/client-grants/this-id', args[0])
        self.assertEqual(kwargs['data'], {'a': 'b', 'c': 'd'})



from .rest import RestClient


class UsersByEmail(object):
    """Auth0 users by email endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self):
        url = '{}://{}/api/v2/users-by-email'.format(self.protocol, self.domain)
        return url

    def search_users_by_email(self, email, fields=None, include_fields=True):
        """List or search users.

        Args:

            email: Email to search.

            fields (list of str, optional): A list of fields to include or
                exclude from the result (depending on include_fields). Leave empty to
                retrieve all fields.

            include_fields (bool, optional): True if the fields specified are
                to be include in the result, False otherwise.

        See: https://auth0.com/docs/api/management/v2#!/Users_By_Email/get_users_by_email
        """
        params = {
            'email': email,
            'fields': fields and ','.join(fields) or None,
            'include_fields': str(include_fields).lower()
        }
        return self.client.get(self._url(), params=params)



from .auth0 import Auth0
from .blacklists import Blacklists
from .client_grants import ClientGrants
from .clients import Clients
from .connections import Connections
from .custom_domains import CustomDomains
from .device_credentials import DeviceCredentials
from .email_templates import EmailTemplates
from .emails import Emails
from .grants import Grants
from .guardian import Guardian
from .hooks import Hooks
from .jobs import Jobs
from .log_streams import LogStreams
from .logs import Logs
from .resource_servers import ResourceServers
from .roles import Roles
from .rules import Rules
from .rules_configs import RulesConfigs
from .stats import Stats
from .tenants import Tenants
from .tickets import Tickets
from .user_blocks import UserBlocks
from .users import Users
from .users_by_email import UsersByEmail



import unittest
import mock
from ...management.rules import Rules


class TestRules(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Rules(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.rules.RestClient')
    def test_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Rules(domain='domain', token='jwttoken')

        # with default params
        c.all()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/rules', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true',
                                            'enabled': 'true',
                                            'stage': 'login_success',
                                            'page': None,
                                            'per_page': None,
                                            'include_totals': 'false'})

        # with stage and fields params
        c.all(stage='stage', enabled=False, fields=['a', 'b'],
              include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/rules', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'false',
                                            'enabled': 'false',
                                            'stage': 'stage',
                                            'page': None,
                                            'per_page': None,
                                            'include_totals': 'false'})

        # with pagination params
        c.all(page=3, per_page=27, include_totals=True)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/rules', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true',
                                            'enabled': 'true',
                                            'stage': 'login_success',
                                            'page': 3,
                                            'per_page': 27,
                                            'include_totals': 'true'})

    @mock.patch('auth0.v3.management.rules.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Rules(domain='domain', token='jwttoken')
        c.create({'a': 'b', 'c': 'd'})

        args, kwargs = mock_instance.post.call_args

        self.assertEqual('https://domain/api/v2/rules', args[0])
        self.assertEqual(kwargs['data'], {'a': 'b', 'c': 'd'})

    @mock.patch('auth0.v3.management.rules.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Rules(domain='domain', token='jwttoken')
        c.get('an-id')

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/rules/an-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true'})

        c.get('an-id', fields=['a', 'b'], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/rules/an-id', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'false'})

    @mock.patch('auth0.v3.management.rules.RestClient')
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Rules(domain='domain', token='jwttoken')
        c.delete('an-id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/rules/an-id'
        )

    @mock.patch('auth0.v3.management.rules.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Rules(domain='domain', token='jwttoken')
        c.update('an-id', {'a': 'b', 'c': 'd'})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual('https://domain/api/v2/rules/an-id', args[0])
        self.assertEqual(kwargs['data'], {'a': 'b', 'c': 'd'})



import unittest
import mock
from ...authentication.delegated import Delegated


class TestDelegated(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.delegated.Delegated.post')
    def test_get_token_id_token(self, mock_post):

        d = Delegated('my.domain.com')

        d.get_token(client_id='cid',
                    target='tgt',
                    api_type='apt',
                    grant_type='gt',
                    id_token='idt',
                    scope='openid profile')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/delegation')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'grant_type': 'gt',
            'id_token': 'idt',
            'target': 'tgt',
            'scope': 'openid profile',
            'api_type': 'apt',
        })

    @mock.patch('auth0.v3.authentication.delegated.Delegated.post')
    def test_get_token_refresh_token(self, mock_post):

        d = Delegated('my.domain.com')

        d.get_token(client_id='cid',
                    target='tgt',
                    api_type='apt',
                    grant_type='gt',
                    refresh_token='rtk')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/delegation')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'grant_type': 'gt',
            'refresh_token': 'rtk',
            'target': 'tgt',
            'scope': 'openid',
            'api_type': 'apt',
        })

    @mock.patch('auth0.v3.authentication.delegated.Delegated.post')
    def test_get_token_value_error(self, mock_post):

        d = Delegated('my.domain.com')

        with self.assertRaises(ValueError):
            d.get_token(client_id='cid',
                        target='tgt',
                        api_type='apt',
                        grant_type='gt',
                        refresh_token='rtk',
                        id_token='idt')



import unittest
import mock
from ...management.logs import Logs


class TestLogs(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Logs(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.logs.RestClient')
    def test_search(self, mock_rc):
        mock_instance = mock_rc.return_value

        logs = Logs(domain='domain', token='jwttoken')
        logs.search()

        args, kwargs = mock_instance.get.call_args
        self.assertEqual('https://domain/api/v2/logs', args[0])
        self.assertIsNone(kwargs['params']['sort'])
        self.assertIsNone(kwargs['params']['q'])
        self.assertIsNone(kwargs['params']['from'])
        self.assertIsNone(kwargs['params']['take'])
        self.assertEqual(kwargs['params']['include_fields'], 'true')
        self.assertEqual(kwargs['params']['include_totals'], 'true')
        self.assertEqual(kwargs['params']['per_page'], 50)
        self.assertEqual(kwargs['params']['page'], 0)
        self.assertIsNone(kwargs['params']['fields'])

        logs.search(fields=['description', 'client_id'])

        args, kwargs = mock_instance.get.call_args
        self.assertEqual(kwargs['params']['fields'], 'description,client_id')

        logs.search(page=0, per_page=2)

        args, kwargs = mock_instance.get.call_args
        self.assertEqual(kwargs['params']['per_page'], 2)
        self.assertEqual(kwargs['params']['page'], 0)

    @mock.patch('auth0.v3.management.logs.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        logs = Logs(domain='domain', token='jwttoken')
        logs.get('get_id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/logs/get_id'
        )



from .rest import RestClient


class UserBlocks(object):
    """Auth0 user blocks endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/user-blocks'.format(self.protocol, self.domain)
        if id is not None:
            return '{}/{}'.format(url, id)
        return url

    def get_by_identifier(self, identifier):
        """Gets blocks by identifier

        Args:
           identifier (str): Should be any of: username, phone_number, email.

        See: https://auth0.com/docs/api/management/v2#!/User_Blocks/get_user_blocks
        """

        params = {'identifier': identifier}

        return self.client.get(self._url(), params=params)

    def unblock_by_identifier(self, identifier):
        """Unblocks by identifier

        Args:
           identifier (str): Should be any of: username, phone_number, email.

        See: https://auth0.com/docs/api/management/v2#!/User_Blocks/delete_user_blocks
        """

        params = {'identifier': identifier}

        return self.client.delete(self._url(), params=params)

    def get(self, id):
        """Get a user's blocks

        Args:
           id (str): The user_id of the user to retrieve.

        See: https://auth0.com/docs/api/management/v2#!/User_Blocks/get_user_blocks_by_id
        """

        return self.client.get(self._url(id))

    def unblock(self, id):
        """Unblock a user

        Args:
           id (str): The user_id of the user to update.

        See: https://auth0.com/docs/api/management/v2#!/User_Blocks/delete_user_blocks_by_id
        """

        return self.client.delete(self._url(id))



import warnings

from .base import AuthenticationBase


class Database(AuthenticationBase):
    """Database & Active Directory / LDAP Authentication.

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def login(self, client_id, username, password, connection, id_token=None,
              grant_type='password', device=None, scope='openid'):
        """Login using username and password

        Given the user credentials and the connection specified, it will do
        the authentication on the provider and return a dict with the
        access_token and id_token. This endpoint only works for database
        connections, passwordless connections, Active Directory/LDAP,
        Windows Azure AD and ADFS.
        """
        warnings.warn("/oauth/ro will be deprecated in future releases", DeprecationWarning)

        body = {
            'client_id': client_id,
            'username': username,
            'password': password,
            'connection': connection,
            'grant_type': grant_type,
            'scope': scope,
        }
        if id_token:
            body.update({'id_token': id_token})
        if device:
            body.update({'device': device})
        return self.post('{}://{}/oauth/ro'.format(self.protocol, self.domain), data=body)

    def signup(self, client_id, email, password, connection, username=None, user_metadata=None,
               given_name=None, family_name=None, name=None, nickname=None, picture=None):
        """Signup using email and password.

        Args:
           client_id (str): ID of the application to use.

           email (str): The user's email address.

           password (str): The user's desired password.

           connection (str): The name of the database connection where this user should be created.

           username (str, optional): The user's username, if required by the database connection.

           user_metadata (dict, optional): Additional key-value information to store for the user.
                    Some limitations apply, see: https://auth0.com/docs/metadata#metadata-restrictions

           given_name (str, optional): The user's given name(s).

           family_name (str, optional): The user's family name(s).

           name (str, optional): The user's full name.

           nickname (str, optional): The user's nickname.

           picture (str, optional): A URI pointing to the user's picture.


        See: https://auth0.com/docs/api/authentication#signup
        """
        body = {
            'client_id': client_id,
            'email': email,
            'password': password,
            'connection': connection,
        }
        if username:
            body.update({'username': username})
        if user_metadata:
            body.update({'user_metadata': user_metadata})
        if given_name:
            body.update({'given_name': given_name})
        if family_name:
            body.update({'family_name': family_name})
        if name:
            body.update({'name': name})
        if nickname:
            body.update({'nickname': nickname})
        if picture:
            body.update({'picture': picture})

        return self.post('{}://{}/dbconnections/signup'.format(self.protocol, self.domain), data=body)

    def change_password(self, client_id, email, connection, password=None):
        """Asks to change a password for a given user.
            
           client_id (str): ID of the application to use.

           email (str): The user's email address.

           connection (str): The name of the database connection where this user should be created.
        """
        body = {
            'client_id': client_id,
            'email': email,
            'connection': connection,
        }
        
        return self.post('{}://{}/dbconnections/change_password'.format(self.protocol, self.domain), data=body)



import unittest
import sys
import json
import base64

import mock
import requests

from ...management.rest import RestClient
from ...exceptions import Auth0Error, RateLimitError


class TestRest(unittest.TestCase):

    def test_get_can_timeout(self):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=0.00001)

        with self.assertRaises(requests.exceptions.Timeout):
            rc.get('http://google.com')

    def test_post_can_timeout(self):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=0.00001)

        with self.assertRaises(requests.exceptions.Timeout):
            rc.post('http://google.com')

    def test_put_can_timeout(self):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=0.00001)

        with self.assertRaises(requests.exceptions.Timeout):
            rc.put('http://google.com')

    def test_patch_can_timeout(self):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=0.00001)

        with self.assertRaises(requests.exceptions.Timeout):
            rc.patch('http://google.com')

    def test_delete_can_timeout(self):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=0.00001)

        with self.assertRaises(requests.exceptions.Timeout):
            rc.delete('http://google.com')

    @mock.patch('requests.get')
    def test_get_custom_timeout(self, mock_get):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=(10, 2))
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }
        mock_get.return_value.text = '["a", "b"]'
        mock_get.return_value.status_code = 200

        rc.get('the-url')
        mock_get.assert_called_with('the-url', params=None, headers=headers, timeout=(10, 2))

    @mock.patch('requests.post')
    def test_post_custom_timeout(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=(10, 2))
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }
        mock_post.return_value.text = '["a", "b"]'
        mock_post.return_value.status_code = 200

        rc.post('the-url')
        mock_post.assert_called_with('the-url', json=None, headers=headers, timeout=(10, 2))

    @mock.patch('requests.put')
    def test_put_custom_timeout(self, mock_put):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=(10, 2))
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }
        mock_put.return_value.text = '["a", "b"]'
        mock_put.return_value.status_code = 200

        rc.put('the-url')
        mock_put.assert_called_with('the-url', json=None, headers=headers, timeout=(10, 2))

    @mock.patch('requests.patch')
    def test_patch_custom_timeout(self, mock_patch):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=(10, 2))
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }
        mock_patch.return_value.text = '["a", "b"]'
        mock_patch.return_value.status_code = 200

        rc.patch('the-url')
        mock_patch.assert_called_with('the-url', json=None, headers=headers, timeout=(10, 2))

    @mock.patch('requests.delete')
    def test_delete_custom_timeout(self, mock_delete):
        rc = RestClient(jwt='a-token', telemetry=False, timeout=(10, 2))
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }
        mock_delete.return_value.text = '["a", "b"]'
        mock_delete.return_value.status_code = 200

        rc.delete('the-url')
        mock_delete.assert_called_with('the-url', params={}, json=None, headers=headers, timeout=(10, 2))

    @mock.patch('requests.get')
    def test_get(self, mock_get):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }

        mock_get.return_value.text = '["a", "b"]'
        mock_get.return_value.status_code = 200

        response = rc.get('the-url')
        mock_get.assert_called_with('the-url', params=None, headers=headers, timeout=5.0)

        self.assertEqual(response, ['a', 'b'])

        response = rc.get(url='the/url', params={'A': 'param', 'B': 'param'})
        mock_get.assert_called_with('the/url', params={'A': 'param',
                                                       'B': 'param'},
                                    headers=headers, timeout=5.0)
        self.assertEqual(response, ['a', 'b'])

        mock_get.return_value.text = ''
        response = rc.get('the/url')
        self.assertEqual(response, '')

    @mock.patch('requests.get')
    def test_get_errors(self, mock_get):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_get.return_value.text = '{"statusCode": 999,' \
                                     ' "errorCode": "code",' \
                                     ' "message": "message"}'
        mock_get.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.get('the/url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')

    @mock.patch('requests.get')
    def test_get_rate_limit_error(self, mock_get):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_get.return_value.text = '{"statusCode": 429,' \
                                     ' "errorCode": "code",' \
                                     ' "message": "message"}'
        mock_get.return_value.status_code = 429
        mock_get.return_value.headers = {
            'x-ratelimit-limit': '3',
            'x-ratelimit-remaining': '6',
            'x-ratelimit-reset': '9',
        }

        with self.assertRaises(Auth0Error) as context:
            rc.get('the/url')

        self.assertEqual(context.exception.status_code, 429)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')
        self.assertIsInstance(context.exception, RateLimitError)
        self.assertEqual(context.exception.reset_at, 9)

    @mock.patch('requests.get')
    def test_get_rate_limit_error_without_headers(self, mock_get):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_get.return_value.text = '{"statusCode": 429,' \
                                     ' "errorCode": "code",' \
                                     ' "message": "message"}'
        mock_get.return_value.status_code = 429
        
        mock_get.return_value.headers = {}
        with self.assertRaises(Auth0Error) as context:
            rc.get('the/url')

        self.assertEqual(context.exception.status_code, 429)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')
        self.assertIsInstance(context.exception, RateLimitError)
        self.assertEqual(context.exception.reset_at, -1)

    @mock.patch('requests.post')
    def test_post(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {'Authorization': 'Bearer a-token',
                   'Content-Type': 'application/json'}

        mock_post.return_value.text = '{"a": "b"}'

        data = {'some': 'data'}

        mock_post.return_value.status_code = 200
        response = rc.post('the/url', data=data)
        mock_post.assert_called_with('the/url', json=data,
                                     headers=headers, timeout=5.0)

        self.assertEqual(response, {'a': 'b'})

    @mock.patch('requests.post')
    def test_post_errors(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_post.return_value.text = '{"statusCode": 999,' \
                                      ' "errorCode": "code",' \
                                      ' "message": "message"}'
        mock_post.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.post('the-url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')

    @mock.patch('requests.post')
    def test_post_errors_with_no_message_property(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_post.return_value.text = json.dumps({
            "statusCode": 999,
            "errorCode": "code",
            "error": "error"
        })
        mock_post.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.post('the-url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'error')

    @mock.patch('requests.post')
    def test_post_errors_with_no_message_or_error_property(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_post.return_value.text = json.dumps({
            "statusCode": 999,
            "errorCode": "code"
        })
        mock_post.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.post('the-url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, '')

    @mock.patch('requests.post')
    def test_post_errors_with_message_and_error_property(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_post.return_value.text = json.dumps({
            "statusCode": 999,
            "errorCode": "code",
            "error": "error",
            "message": "message"
        })
        mock_post.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.post('the-url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')

    @mock.patch('requests.post')
    def test_post_error_with_code_property(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        for error_status in [400, 500, None]:
            mock_post.return_value.status_code = error_status
            mock_post.return_value.text = '{"errorCode": "e0",' \
                                          '"message": "desc"}'

            with self.assertRaises(Auth0Error) as context:
                rc.post('the-url')

            self.assertEqual(context.exception.status_code, error_status)
            self.assertEqual(context.exception.error_code, 'e0')
            self.assertEqual(context.exception.message, 'desc')

    @mock.patch('requests.post')
    def test_post_error_with_no_error_code(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        for error_status in [400, 500, None]:
            mock_post.return_value.status_code = error_status
            mock_post.return_value.text = '{"message": "desc"}'

            with self.assertRaises(Auth0Error) as context:
                rc.post('the-url')

            self.assertEqual(context.exception.status_code, error_status)
            self.assertEqual(context.exception.error_code, 'a0.sdk.internal.unknown')
            self.assertEqual(context.exception.message, 'desc')

    @mock.patch('requests.post')
    def test_post_error_with_text_response(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        for error_status in [400, 500, None]:
            mock_post.return_value.status_code = error_status
            mock_post.return_value.text = 'there has been a terrible error'

            with self.assertRaises(Auth0Error) as context:
                rc.post('the-url')

            self.assertEqual(context.exception.status_code, error_status)
            self.assertEqual(context.exception.error_code, 'a0.sdk.internal.unknown')
            self.assertEqual(context.exception.message,
                             'there has been a terrible error')

    @mock.patch('requests.post')
    def test_post_error_with_no_response_text(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)

        for error_status in [400, 500, None]:
            mock_post.return_value.status_code = error_status
            mock_post.return_value.text = None

            with self.assertRaises(Auth0Error) as context:
                rc.post('the-url')

            self.assertEqual(context.exception.status_code, error_status)
            self.assertEqual(context.exception.error_code, 'a0.sdk.internal.unknown')
            self.assertEqual(context.exception.message, '')

    @mock.patch('requests.post')
    def test_file_post_content_type_is_none(self, mock_post):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {'Authorization': 'Bearer a-token'}
        mock_post.return_value.status_code = 200
        mock_post.return_value.text = 'Success'

        data = {'some': 'data'}
        files = [mock.Mock()]

        rc.file_post('the-url', data=data, files=files)

        mock_post.assert_called_once_with('the-url', data=data, files=files, headers=headers, timeout=5.0)

    @mock.patch('requests.put')
    def test_put(self, mock_put):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {'Authorization': 'Bearer a-token',
                   'Content-Type': 'application/json'}

        mock_put.return_value.text = '["a", "b"]'
        mock_put.return_value.status_code = 200

        data = {'some': 'data'}

        response = rc.put(url='the-url', data=data)
        mock_put.assert_called_with('the-url', json=data,
                                    headers=headers, timeout=5.0)

        self.assertEqual(response, ['a', 'b'])

    @mock.patch('requests.put')
    def test_put_errors(self, mock_put):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_put.return_value.text = '{"statusCode": 999,' \
                                     ' "errorCode": "code",' \
                                     ' "message": "message"}'
        mock_put.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.put(url='the/url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')

    @mock.patch('requests.patch')
    def test_patch(self, mock_patch):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {'Authorization': 'Bearer a-token',
                   'Content-Type': 'application/json'}

        mock_patch.return_value.text = '["a", "b"]'
        mock_patch.return_value.status_code = 200

        data = {'some': 'data'}

        response = rc.patch(url='the-url', data=data)
        mock_patch.assert_called_with('the-url', json=data,
                                      headers=headers, timeout=5.0)

        self.assertEqual(response, ['a', 'b'])

    @mock.patch('requests.patch')
    def test_patch_errors(self, mock_patch):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_patch.return_value.text = '{"statusCode": 999,' \
                                       ' "errorCode": "code",' \
                                       ' "message": "message"}'
        mock_patch.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.patch(url='the/url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')

    @mock.patch('requests.delete')
    def test_delete(self, mock_delete):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }

        mock_delete.return_value.text = '["a", "b"]'
        mock_delete.return_value.status_code = 200

        response = rc.delete(url='the-url/ID')
        mock_delete.assert_called_with('the-url/ID', headers=headers, params={}, json=None, timeout=5.0)

        self.assertEqual(response, ['a', 'b'])

    @mock.patch('requests.delete')
    def test_delete_with_body_and_params(self, mock_delete):
        rc = RestClient(jwt='a-token', telemetry=False)
        headers = {
            'Authorization': 'Bearer a-token',
            'Content-Type': 'application/json',
        }

        mock_delete.return_value.text = '["a", "b"]'
        mock_delete.return_value.status_code = 200

        data = {'some': 'data'}
        params = {'A': 'param', 'B': 'param'}

        response = rc.delete(url='the-url/ID', params=params, data=data)
        mock_delete.assert_called_with('the-url/ID', headers=headers, params=params, json=data, timeout=5.0)

        self.assertEqual(response, ['a', 'b'])

    @mock.patch('requests.delete')
    def test_delete_errors(self, mock_delete):
        rc = RestClient(jwt='a-token', telemetry=False)

        mock_delete.return_value.text = '{"statusCode": 999,' \
                                        ' "errorCode": "code",' \
                                        ' "message": "message"}'
        mock_delete.return_value.status_code = 999

        with self.assertRaises(Auth0Error) as context:
            rc.delete(url='the-url')

        self.assertEqual(context.exception.status_code, 999)
        self.assertEqual(context.exception.error_code, 'code')
        self.assertEqual(context.exception.message, 'message')

    def test_disabled_telemetry(self):
        rc = RestClient(jwt='a-token', telemetry=False)
        expected_headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer a-token',
        }

        self.assertEqual(rc.base_headers, expected_headers)

    def test_enabled_telemetry(self):
        rc = RestClient(jwt='a-token', telemetry=True)

        user_agent = rc.base_headers['User-Agent']
        auth0_client_bytes = base64.b64decode(rc.base_headers['Auth0-Client'])
        auth0_client_json = auth0_client_bytes.decode('utf-8')
        auth0_client = json.loads(auth0_client_json)
        content_type = rc.base_headers['Content-Type']

        from auth0 import __version__ as auth0_version
        python_version = '{}.{}.{}'.format(sys.version_info.major,
                                           sys.version_info.minor,
                                           sys.version_info.micro)

        client_info = {
            'name': 'auth0-python',
            'version': auth0_version,
            'env': {
                'python': python_version
            }
        }

        self.assertEqual(user_agent, 'Python/{}'.format(python_version))
        self.assertEqual(auth0_client, client_info)
        self.assertEqual(content_type, 'application/json')



from .rest import RestClient


class RulesConfigs(object):
    """RulesConfig endpoint implementation.

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/rules-configs'.format(self.protocol, self.domain)
        if id is not None:
            return url + '/' + id
        return url

    def all(self):
        """Lists the config variable keys for rules.

        See: https://auth0.com/docs/api/management/v2#!/Rules_Configs/get_rules_configs
        """
        return self.client.get(self._url())

    def unset(self, key):
        """Removes the rules config for a given key.

        Args:
            key (str): rules config key to remove.

        See: https://auth0.com/docs/api/management/v2#!/Rules_Configs/delete_rules_configs_by_key
        """
        return self.client.delete(self._url(key))

    def set(self, key, value):
        """Sets the rules config for a given key.

        Args:
            key (str): rules config key to set.

            value (str): value to set for the rules config key.

        See: https://auth0.com/docs/api/management/v2#!/Rules_Configs/put_rules_configs_by_key
        """
        url = self._url('{}'.format(key))
        body = {'value': value}
        return self.client.put(url, data=body)



from .base import AuthenticationBase


class AuthorizeClient(AuthenticationBase):

    """Authorize Client

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def authorize(self, client_id, audience=None, state=None, redirect_uri=None,
                  response_type='code', scope='openid', organization=None, invitation=None):
        """Authorization code grant

        This is the OAuth 2.0 grant that regular web apps utilize in order to access an API.
        """
        params = {
            'client_id': client_id,
            'audience': audience,
            'response_type': response_type,
            'scope': scope,
            'state': state,
            'redirect_uri': redirect_uri,
            'organization': organization,
            'invitation': invitation
        }

        return self.get(
            '{}://{}/authorize'.format(self.protocol, self.domain),
            params=params)



from .base import AuthenticationBase


class RevokeToken(AuthenticationBase):
    """Revoke Refresh Token endpoint

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def revoke_refresh_token(self, client_id, token, client_secret=None):
        """Revokes a Refresh Token if it has been compromised
           
           Each revocation request invalidates not only the specific token, but all other tokens 
           based on the same authorization grant. This means that all Refresh Tokens that have 
           been issued for the same user, application, and audience will be revoked.

           Args:
                client_id (str): The Client ID for your Application

                token (str): The Refresh Token you want to revoke

                client_secret (str, optional): The Client Secret for your Application.
                        Required for confidential applications.
                        See: https://auth0.com/docs/applications/application-types#confidential-applications
            
            See: https://auth0.com/docs/api/authentication#refresh-token
        """
        body = {
            'client_id': client_id,
            'token': token,
        }

        if client_secret:
            body.update({'client_secret': client_secret})

        return self.post('{}://{}/oauth/revoke'.format(self.protocol, self.domain), data=body)



import unittest
import mock
from ...management.jobs import Jobs


class TestJobs(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Jobs(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.jobs.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        j = Jobs(domain='domain', token='jwttoken')
        j.get('an-id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/jobs/an-id',
        )

    @mock.patch('auth0.v3.management.jobs.RestClient')
    def get_failed_job(self, mock_rc):
        mock_instance = mock_rc.return_value

        j = Jobs(domain='domain', token='jwttoken')
        j.get('an-id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/jobs/an-id/errors',
        )

    @mock.patch('auth0.v3.management.jobs.RestClient')
    def get_job_results(self, mock_rc):
        mock_instance = mock_rc.return_value

        j = Jobs(domain='domain', token='jwttoken')
        j.get('an-id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/jobs/an-id/results',
        )

    @mock.patch('auth0.v3.management.jobs.RestClient')
    def test_export_users(self, mock_rc):
        mock_instance = mock_rc.return_value

        j = Jobs(domain='domain', token='jwttoken')
        j.export_users({'connection_id': 'cxn_id', 'format': 'json'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/jobs/users-exports',
            data={'connection_id': 'cxn_id', 'format': 'json'}
        )

    @mock.patch('auth0.v3.management.jobs.RestClient')
    def test_import_users(self, mock_rc):
        mock_instance = mock_rc.return_value

        j = Jobs(domain='domain', token='jwttoken')
        j.import_users(connection_id='1234', file_obj={})

        mock_instance.file_post.assert_called_with(
            'https://domain/api/v2/jobs/users-imports',
            data={'connection_id': '1234', 'upsert': 'false', 'send_completion_email': 'true', 'external_id': None},
            files={'users': {}}
        )

        j.import_users(connection_id='1234', file_obj={}, upsert=True, send_completion_email=False, external_id="ext-id-123")
        mock_instance.file_post.assert_called_with(
            'https://domain/api/v2/jobs/users-imports',
            data={'connection_id': '1234', 'upsert': 'true', 'send_completion_email': 'false', 'external_id': 'ext-id-123'},
            files={'users': {}}
        )

        j.import_users(connection_id='1234', file_obj={}, upsert=False, send_completion_email=True)
        mock_instance.file_post.assert_called_with(
            'https://domain/api/v2/jobs/users-imports',
            data={'connection_id': '1234', 'upsert': 'false', 'send_completion_email': 'true', 'external_id': None},
            files={'users': {}}
        )

    @mock.patch('auth0.v3.management.jobs.RestClient')
    def test_verification_email(self, mock_rc):
        mock_instance = mock_rc.return_value

        j = Jobs(domain='domain', token='jwttoken')
        j.send_verification_email({'a': 'b', 'c': 'd'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/jobs/verification-email',
            data={'a': 'b', 'c': 'd'}
        )



import unittest
import mock
from ...management.stats import Stats


class TestStats(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Stats(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.stats.RestClient')
    def test_active_users(self, mock_rc):
        mock_instance = mock_rc.return_value

        s = Stats(domain='domain', token='jwttoken')
        s.active_users()

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/stats/active-users',
        )

    @mock.patch('auth0.v3.management.stats.RestClient')
    def test_daily_stats(self, mock_rc):
        mock_instance = mock_rc.return_value

        s = Stats(domain='domain', token='jwttoken')
        s.daily_stats()

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/stats/daily',
            params={'from': None, 'to': None},
        )

        s.daily_stats(from_date='12341212', to_date='56785656')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/stats/daily',
            params={'from': '12341212', 'to': '56785656'},
        )



from .rest import RestClient


class Organizations(object):
    """Auth0 organizations endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, *args):
        url = '{}://{}/api/v2/organizations'.format(self.protocol, self.domain)
        for p in args:
            if p is not None:
                url = '{}/{}'.format(url, p)
        return url

    # Organizations
    def all_organizations(self, page=None, per_page=None):
        """Retrieves a list of all the organizations.

        Args:
           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_organizations
        """
        params = {}
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(), params=params)

    def get_organization_by_name(self, name=None):
        """Retrieves an organization given its name.

        Args:
           name (str): The name of the organization to retrieve.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_name_by_name
        """
        params = {}

        return self.client.get(self._url('name', name), params=params)

    def get_organization(self, id):
        """Retrieves an organization by its ID.

        Args:
           id (str): Id of organization to retrieve.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_organizations_by_id
        """
        params = {}

        return self.client.get(self._url(id), params=params)

    def create_organization(self, body):
        """Create a new organization.

        Args:
           body (dict): Attributes for the new organization.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_organizations
        """

        return self.client.post(self._url(), data=body)
    
    def update_organization(self, id, body):
        """Modifies an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/patch_organizations_by_id
        """

        return self.client.patch(self._url(id), data=body)

    def delete_organization(self, id):
        """Deletes an organization and all its related assets.

        Args:
           id (str): Id of organization to delete.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_organizations_by_id
        """

        return self.client.delete(self._url(id))


    # Organization Connections
    def all_organization_connections(self, id, page=None, per_page=None):
        """Retrieves a list of all the organization connections.

        Args:
           id (str): the ID of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_enabled_connections
        """
        params = {}
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(id, 'enabled_connections'), params=params)

    def get_organization_connection(self, id, connection_id):
        """Retrieves an organization connection by its ID.

        Args:
           id (str): the ID of the organization.

           connection_id (str): the ID of the connection.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_enabled_connections_by_connectionId
        """
        params = {}

        return self.client.get(self._url(id, 'enabled_connections', connection_id), params=params)

    def create_organization_connection(self, id, body):
        """Adds a connection to an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes for the connection to add.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_enabled_connections
        """

        return self.client.post(self._url(id, 'enabled_connections'), data=body)
    
    def update_organization_connection(self, id, connection_id, body):
        """Modifies an organization.

        Args:
           id (str): the ID of the organization.

           connection_id (str): the ID of the connection to update.

           body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/patch_enabled_connections_by_connectionId
        """

        return self.client.patch(self._url(id, 'enabled_connections', connection_id), data=body)

    def delete_organization_connection(self, id, connection_id):
        """Deletes a connection from the given organization.

        Args:
           id (str): Id of organization.

           connection_id (str): the ID of the connection to delete.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_enabled_connections_by_connectionId
        """

        return self.client.delete(self._url(id, 'enabled_connections', connection_id))

    # Organization Members
    def all_organization_members(self, id, page=None, per_page=None):
        """Retrieves a list of all the organization members.

        Args:
           id (str): the ID of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_members
        """
        params = {}
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(id, 'members'), params=params)

    def create_organization_members(self, id, body):
        """Adds members to an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes from the members to add.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_members
        """

        return self.client.post(self._url(id, 'members'), data=body)

    def delete_organization_members(self, id, body):
        """Deletes members from the given organization.

        Args:
           id (str): Id of organization.

           body (dict): Attributes from the members to delete

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_members
        """

        return self.client.delete(self._url(id, 'members'), data=body)

    # Organization Member Roles
    def all_organization_member_roles(self, id, user_id, page=None, per_page=None):
        """Retrieves a list of all the roles from the given organization member.

        Args:
           id (str): the ID of the organization.
           
           user_id (str): the ID of the user member of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_organization_member_roles
        """
        params = {}
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(id, 'members', user_id, 'roles'), params=params)

    def create_organization_member_roles(self, id, user_id, body):
        """Adds roles to a member of an organization.

        Args:
           id (str): the ID of the organization.

           user_id (str): the ID of the user member of the organization.

           body (dict): Attributes from the members to add.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_organization_member_roles
        """

        return self.client.post(self._url(id, 'members', user_id, 'roles'), data=body)

    def delete_organization_member_roles(self, id, user_id, body):
        """Deletes roles from a member of an organization.

        Args:
           id (str): Id of organization.

           user_id (str): the ID of the user member of the organization.

           body (dict): Attributes from the members to delete

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_organization_member_roles
        """

        return self.client.delete(self._url(id, 'members', user_id, 'roles'), data=body)


    # Organization Invitations
    def all_organization_invitations(self, id, page=None, per_page=None):
        """Retrieves a list of all the organization invitations.

        Args:
           id (str): the ID of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_invitations
        """
        params = {}
        params['page'] = page
        params['per_page'] = per_page

        return self.client.get(self._url(id, 'invitations'), params=params)

    def get_organization_invitation(self, id, invitaton_id):
        """Retrieves an organization invitation by its ID.

        Args:
           id (str): the ID of the organization.

           invitaton_id (str): the ID of the invitation.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_invitations_by_invitation_id
        """
        params = {}

        return self.client.get(self._url(id, 'invitations', invitaton_id), params=params)

    def create_organization_invitation(self, id, body):
        """Create an invitation to an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes for the invitation to create.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_invitations
        """

        return self.client.post(self._url(id, 'invitations'), data=body)
   
    def delete_organization_invitation(self, id, invitation_id):
        """Deletes an invitation from the given organization.

        Args:
           id (str): Id of organization.

           invitation_id (str): the ID of the invitation to delete.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_invitations_by_invitation_id
        """

        return self.client.delete(self._url(id, 'invitations', invitation_id))



import unittest
import mock
from ...management.user_blocks import UserBlocks


class TestUserBlocks(unittest.TestCase):

    def test_init_with_optionals(self):
        t = UserBlocks(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.user_blocks.RestClient')
    def test_get_by_identifier(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain='domain', token='jwttoken')

        u.get_by_identifier('some_identifier')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/user-blocks',
            params={'identifier': 'some_identifier'}
        )

    @mock.patch('auth0.v3.management.user_blocks.RestClient')
    def test_unblock_by_identifier(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain='domain', token='jwttoken')

        u.unblock_by_identifier('test@test.com')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/user-blocks',
            params={'identifier': 'test@test.com'}
        )

    @mock.patch('auth0.v3.management.user_blocks.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain='domain', token='jwttoken')

        u.get('auth0|584ad3c228be27504a2c80d5')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/user-blocks/auth0|584ad3c228be27504a2c80d5'
        )

    @mock.patch('auth0.v3.management.user_blocks.RestClient')
    def test_unblock(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain='domain', token='jwttoken')

        u.unblock('auth0|584ad3c228be27504a2c80d5')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/user-blocks/auth0|584ad3c228be27504a2c80d5'
        )



import unittest
import mock
from ...management.tenants import Tenants


class TestTenants(unittest.TestCase):

    def test_init_with_optionals(self):
        t = Tenants(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.tenants.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        t = Tenants(domain='domain', token='jwttoken')
        t.get()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/tenants/settings', args[0])
        self.assertEqual(kwargs['params'], {'fields': None,
                                            'include_fields': 'true'})

        t.get(fields=['a', 'b'], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/tenants/settings', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'false'})

        t.get(fields=['a', 'b'], include_fields=True)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual('https://domain/api/v2/tenants/settings', args[0])
        self.assertEqual(kwargs['params'], {'fields': 'a,b',
                                            'include_fields': 'true'})

    @mock.patch('auth0.v3.management.tenants.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.patch.return_value = {}

        t = Tenants(domain='domain', token='jwttoken')
        t.update({'a': 'b', 'c': 'd'})

        mock_instance.patch.assert_called_with(
            'https://domain/api/v2/tenants/settings',
            data={'a': 'b', 'c': 'd'}
        )



import unittest
import mock
from ...management.resource_servers import ResourceServers


class TestResourceServers(unittest.TestCase):

    def test_init_with_optionals(self):
        t = ResourceServers(domain='domain', token='jwttoken', telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get('Auth0-Client', None)
        self.assertEqual(telemetry_header, None)

    @mock.patch('auth0.v3.management.resource_servers.RestClient')
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain='domain', token='jwttoken')

        r.create({'name': 'TestApi', 'identifier': 'https://test.com/api'})

        mock_instance.post.assert_called_with(
            'https://domain/api/v2/resource-servers',
            data={'name': 'TestApi', 'identifier': 'https://test.com/api'}
        )

    @mock.patch('auth0.v3.management.resource_servers.RestClient')
    def test_get_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain='domain', token='jwttoken')

        # with default params
        r.get_all()

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/resource-servers',
            params={
                'page': None,
                'per_page': None,
                'include_totals': 'false'
            }
        )

        # with pagination params
        r.get_all(page=3, per_page=27, include_totals=True)

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/resource-servers',
            params={
                'page': 3,
                'per_page': 27,
                'include_totals': 'true'
            }
        )

    @mock.patch('auth0.v3.management.resource_servers.RestClient')
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain='domain', token='jwttoken')

        r.get('some_id')

        mock_instance.get.assert_called_with(
            'https://domain/api/v2/resource-servers/some_id'
        )

    @mock.patch('auth0.v3.management.resource_servers.RestClient')
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain='domain', token='jwttoken')

        r.delete('some_id')

        mock_instance.delete.assert_called_with(
            'https://domain/api/v2/resource-servers/some_id'
        )

    @mock.patch('auth0.v3.management.resource_servers.RestClient')
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain='domain', token='jwttoken')

        r.update('some_id', {'name': 'TestApi2',
                             'identifier': 'https://test.com/api2'})

        mock_instance.patch.assert_called_with(
            'https://domain/api/v2/resource-servers/some_id',
            data={'name': 'TestApi2',
                  'identifier': 'https://test.com/api2'}
        )



import unittest
import mock
from ...authentication.get_token import GetToken


class TestGetToken(unittest.TestCase):

    @mock.patch('auth0.v3.authentication.get_token.GetToken.post')
    def test_authorization_code(self, mock_post):

        g = GetToken('my.domain.com')

        g.authorization_code(client_id='cid',
                             client_secret='clsec',
                             code='cd',
                             grant_type='gt',
                             redirect_uri='idt')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/token')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'client_secret': 'clsec',
            'code': 'cd',
            'grant_type': 'gt',
            'redirect_uri': 'idt'
        })

    @mock.patch('auth0.v3.authentication.get_token.GetToken.post')
    def test_authorization_code_pkce(self, mock_post):

        g = GetToken('my.domain.com')

        g.authorization_code_pkce(client_id='cid',
                                  code_verifier='cdver',
                                  code='cd',
                                  grant_type='gt',
                                  redirect_uri='idt')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/token')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'code_verifier': 'cdver',
            'code': 'cd',
            'grant_type': 'gt',
            'redirect_uri': 'idt'
        })

    @mock.patch('auth0.v3.authentication.get_token.GetToken.post')
    def test_client_credentials(self, mock_post):

        g = GetToken('my.domain.com')

        g.client_credentials(client_id='cid',
                             client_secret='clsec',
                             audience='aud',
                             grant_type='gt')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/token')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'client_secret': 'clsec',
            'audience': 'aud',
            'grant_type': 'gt'
        })

    @mock.patch('auth0.v3.authentication.get_token.GetToken.post')
    def test_login(self, mock_post):

        g = GetToken('my.domain.com')

        g.login(client_id='cid',
                client_secret='clsec',
                username='usrnm',
                password='pswd',
                scope='http://test.com/api',
                realm='rlm',
                audience='aud',
                grant_type='gt')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/token')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'client_secret': 'clsec',
            'username': 'usrnm',
            'password': 'pswd',
            'scope': 'http://test.com/api',
            'realm': 'rlm',
            'audience': 'aud',
            'grant_type': 'gt'
        })

    @mock.patch('auth0.v3.authentication.get_token.GetToken.post')
    def test_refresh_token(self, mock_post):
        g = GetToken('my.domain.com')

        g.refresh_token(client_id='cid',
                        client_secret='clsec',
                        refresh_token='rt',
                        grant_type='gt',
                        scope='s')

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], 'https://my.domain.com/oauth/token')
        self.assertEqual(kwargs['data'], {
            'client_id': 'cid',
            'client_secret': 'clsec',
            'refresh_token': 'rt',
            'grant_type': 'gt',
            'scope': 's'
        })



import io
import os
import re
from setuptools import setup, find_packages


def find_version():
    file_dir = os.path.dirname(__file__)
    with io.open(os.path.join(file_dir, 'auth0', '__init__.py')) as f:
        version = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', f.read())
        if version:
            return version.group(1)
        else:
            raise RuntimeError("Unable to find version string.")


with io.open('README.rst', encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='auth0-python',
    version=find_version(),
    description='Auth0 Python SDK',
    long_description=long_description,
    author='Auth0',
    author_email='support@auth0.com',
    license='MIT',
    packages=find_packages(),
    install_requires=['requests>=2.14.0', 'pyjwt[crypto]>=1.7.1'],
    extras_require={'test': ['mock>=1.3.0', 'pre-commit']},
    python_requires='>=2.7, !=3.0.*, !=3.1.*',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    url='https://github.com/auth0/auth0-python',
)



from .rest import RestClient


class DeviceCredentials(object):
    """Auth0 connection endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)
    """

    def __init__(self, domain, token, telemetry=True, timeout=5.0, protocol="https"):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(jwt=token, telemetry=telemetry, timeout=timeout)

    def _url(self, id=None):
        url = '{}://{}/api/v2/device-credentials'.format(self.protocol, self.domain)
        if id is not None:
            return '{}/{}'.format(url, id)
        return url

    def get(self, user_id, client_id, type, fields=None, include_fields=True):
        """List device credentials.

        Args:
            user_id (str): The user_id of the devices to retrieve.

            client_id (str): The client_id of the devices to retrieve.

            type (str): The type of credentials (public_key, refresh_token).

            fields (list, optional): A list of fields to include or exclude
                (depending on include_fields) from the result. Leave empty to
                retrieve all fields.

            include_fields (bool, optional): True if the fields specified are
                to be included in the result, False otherwise. Defaults to True.

        See: https://auth0.com/docs/api/management/v2#!/Device_Credentials/get_device_credentials
        """

        params = {
            'fields': fields and ','.join(fields) or None,
            'include_fields': str(include_fields).lower(),
            'user_id': user_id,
            'client_id': client_id,
            'type': type,
        }
        return self.client.get(self._url(), params=params)

    def create(self, body):
        """Create a device public key.

        Args:
            body (dict): parameters for creating the public key (e.g: type,
                device_name, client_id, etc).

        See: https://auth0.com/docs/api/v2#!/Device_Credentials/post_device_credentials
        """
        return self.client.post(self._url(), data=body)

    def delete(self, id):
        """Delete credential.

        Args:
            id (str):  The id of the credential to delete.

        See: https://auth0.com/docs/api/management/v2#!/Device_Credentials/delete_device_credentials_by_id
        """
        return self.client.delete(self._url(id))
